form.automatic-assignment {
    max-width: 700px;
}

.sortable-box,
.sortable-placeholder {
    border-style: solid;
    border-width: 1.5px;
    margin: 5px;
    border-color: #aaa;
    background-color: #eee;
}

.sortable-placeholder {
    border-style: dashed;
}

.sortable-box .inner {
    padding: 5px;
}

.sortable-box h3 {
    border-bottom: 1px solid #ccc;
    margin: 0;
    padding: 5px 0 0 5px;
}

.sortable-box select {
    min-width: 10em;
}

.replacing .sortable-box {
    -webkit-filter: grayscale(80%);
    filter: grayscale(80%);
    opacity: 0.5;
}

.filters .loading,
.chooser .loading {
    visibility: hidden;
}

.filters.adding .loading,
.chooser.replacing .loading {
    visibility: visible;
}

.sortable-box .remove {
    float: right;
}

.sortable-box .remove img {
    height: 1em;
    width: 1em;
}

form.automatic-assignment textarea {
    font-family: monospace;
}
